﻿namespace DeepDiveBinomial

module Listing12 =

   let Binomial x n p =

      let f = float

      let mutable totalUnscaledP = 0.
      let mutable unscaledResult = 0.
      let essentiallyZero = 10.E-12
      let m = (f(n) * p) |> truncate |> int

      totalUnscaledP <- totalUnscaledP + 1.

      let mutable unscaledResult =
         if m <= x then 1.
         else 0.

      let mutable previousValue = 1.
      let mutable isDone = false

      let mutable k = m + 1

      while not isDone && k <= n do
         let currentValue = previousValue * f(n - k + 1) * p / (f(k) * (1. - p))
         totalUnscaledP <- totalUnscaledP + currentValue
         if k <= x then unscaledResult <- unscaledResult + currentValue
         if currentValue <= essentiallyZero then isDone <- true
         previousValue <- currentValue
         k <- k + 1

      previousValue <- 1.
      isDone <- false
      k <- m - 1
 
      while not isDone && k >= 0 do
         let currentValue = previousValue * f(k + 1) * (1. - p) / (f(n - k) * p)
         totalUnscaledP <- totalUnscaledP + currentValue
         if k <= x then unscaledResult <- unscaledResult + currentValue
         if currentValue <= essentiallyZero then isDone <- true
         previousValue <- currentValue
         k <- k - 1

      unscaledResult / totalUnscaledP

   open NUnit.Framework
   open FsUnit

   // In reality you'd keep these these tests in their own project and source file
   // and refactor the Binomial code without changing the tests
   [<NUnit.Framework.TestFixture>]
   type ``Given the Binomial function``() =

      [<TestCase(1,10,0.5,0.0107421875)>]
      [<TestCase(2,10,0.5,0.0546875)>]
      [<TestCase(3,10,0.5,0.171875)>]
      [<TestCase(4,10,0.5,0.376953125)>]
      [<TestCase(5,10,0.5,0.623046875)>]
      [<TestCase(5,11,0.5,0.5)>]
      [<TestCase(5,12,0.5,0.38720703125)>]
      [<TestCase(5,13,0.5,0.29052734375)>]
      [<TestCase(5,14,0.5,0.21197509765625)>]
      [<TestCase(5,20000,0.001,7.14974861864806E-05)>]
      [<TestCase(6,20000,0.002,2.74846740618111E-11)>]
      [<TestCase(7,20000,0.003,5.11871527472253E-18)>]
      [<TestCase(179,12798,0.012013,0.9797611704)>]
      [<TestCase(12798,12798,0.012013,1.0)>]
      member t.``the result is calculated correctly``(x, n, p, expected) =
         let errorMargin = 10E-9
         let actual = Binomial x n p
         actual |> should (equalWithin errorMargin) expected